declare type Func = (...args: any) => any | Promise<any>;
/**
 * The base of a Monad
 */
declare class MonadBase<T> {
    value: T;
    log: any[] | null | undefined;
    constructor(value?: any, log?: any[] | null);
    apply(func: Func, ...args: any): MonadBase<T>;
    protected finish(skip?: boolean): MonadBase<T>;
    promise(func: Func, ...args: any): PromisedMonad<T>;
    static all<T>(promisedMonads: PromisedMonad<any>[]): Promise<Monad<T>[]>;
}
/**
 * Maybe extension
 */
declare class Maybe<M> extends MonadBase<M> {
    private before;
    constructor(value?: any, log?: any[] | null, before?: Func);
    apply(func: Func, ...args: any): Maybe<M>;
}
/**
 * Either extension
 */
declare class Either<E> extends MonadBase<E> {
    apply(leftFunc: Func, rightFunc: Func, ...args: any): Either<E>;
    fold(maybe?: boolean): MonadBase<E>;
}
/**
 * Promise extension
 */
declare class PromisedMonad<T> extends MonadBase<T> {
    private tasks;
    constructor(maybe: MonadBase<T>, tasks: [Func, any][]);
    promise(func: Func, ...args: any): this;
    resolve(): Promise<MonadBase<T>>;
}
export default class Monad<T> extends MonadBase<T> {
    static Maybe: typeof Maybe;
    static Either: typeof Either;
}
export {};
